// Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>, see README.TXT for details

#include <iostream>
#include <cstdlib>
#include <ctime>
#include <cstdio>

#include "pingu.hh"
#include "colmap.hh"

vector<CL_Surface*> Pingu::pingu;
//CL_Surface* Pingu::fall_sprite = 0; 
//CL_Surface* Pingu::explo = 0;
ColMap*     Pingu::colmap = 0;
PinguMap*   Pingu::map = 0;
//CL_Surface* Pingu::walker;

Pingu::Pingu(int x, int y)
{
  alive = true;
  falling = 0;
  direction = left;
  blocker_id = -1;
  anim_pos.set_size(8);
  set_pos(x, y);
  falling = 0;
}

Pingu::Pingu()
{
  alive = true;
  blocker_id = -1;
  falling = 0;
  direction = left;
  anim_pos.set_size(8);
}

Pingu::~Pingu()
{
}

void
Pingu::get_surface()
{
  
}

void
Pingu::restore_surface()
{
  
}

void
Pingu::draw(int x, int y)
{
  //  restore_surface();
  //  get_surface();
  
  if (!alive) {
    pingu[dead]->put_screen(x,y);
    return;
  }

  if (action == exploder) {
    pingu[exploder]->put_screen(x,y);
    return;
  }

  if (action == blocker) {
    pingu[blocker]->put_screen(x, y, anim_pos.get_value());
    anim_pos.inc();     
    return;
  }

  if (action == floater) {
    pingu[floater]->put_screen(x, y, anim_pos.get_value());
    anim_pos.inc();
    return;
  }
  
  if (falling > 2) {
           
    if (pingu[faller]) {
      pingu[faller]->put_screen(x, y);
    } else {
      cout << "aeuaoeuaoeuaoeuoae" << endl;
      exit(EXIT_FAILURE);
    }
  } else {

    pingu[walker]->put_screen(x, y, anim_pos.get_value()
			      + ((direction == -1) ? 0 : 8 )
			      );
    
    anim_pos.inc();
  }
}

// simple wraper function around draw(x, y)
void
Pingu::draw()
{
  draw(x_pos-16, y_pos-32);
}

void 
Pingu::set_pos(int x, int y)
{
  x_pos = x;
  y_pos = y;
}

void change(int& dir)
{
  if (dir == left)
    dir = right;
  else
    dir =left;
}

// Checks if the pixel before the pingu are walkable
int
Pingu::rel_getpixel(int x, int y)
{
  if (colmap == 0) {
    cout << "Colmap nicht benutzt" << endl;
    exit(EXIT_FAILURE);
  }

  int return_value;

  return_value = colmap->getpixel(x_pos + (x * direction), (y_pos) - y);

  return return_value;
}


void
Pingu::let_move(void)
{
  if (!alive || action == blocker) {
    return;
  }
  
  if (rel_getpixel(0, -1) == 0) {
    // Pingu is falling
    ++y_pos;
    if (rel_getpixel(0, -1) == 0 && action != floater)
      ++y_pos;
    ++falling;
    
  } else {
    // If Pingu hit the ground after 100 pixel, he is dead
    if (falling > 100 && action != floater) {
      alive = false;
      return;
    }
    action = walker;
    
    // Pingu is walking
    falling = 0;
    if (rel_getpixel(1, 0) == 0) {
      x_pos += direction;    
    } else {
      // Pingu is walking mounten up/down
      if (rel_getpixel(1,1)== 0) {
	x_pos += direction;
	y_pos--;
      } else if (rel_getpixel(1,2)== 0) {
	x_pos += direction;
	y_pos -= 2;
      } else {
	change(direction);
      }
    }
  }
}

bool
Pingu::is_alive(void)
{
  return alive;
}

void
Pingu::set_colmap(ColMap* a)
{
  cout << "Colmap gesetzt: " << a << endl;
  colmap = a;
}

void
Pingu::set_map(PinguMap* m)
{
  map = m;
}

void
Pingu::load_data(CL_ResourceManager* resource)
{
  cout << "Loading Pingu Data..." << endl;
  pingu.resize(15);

  pingu[walker]    = CL_Res_Surface::load("Pingus/walker", *resource);
  pingu[faller]    = CL_Res_Surface::load("Pingus/faller", *resource);
  pingu[exploder]  = CL_Res_Surface::load("Pingus/explo", *resource);
  pingu[blocker]   = CL_Res_Surface::load("Pingus/blocker", *resource);
  pingu[floater]   = CL_Res_Surface::load("Pingus/floater", *resource);
  pingu[bridger]   = CL_Res_Surface::load("Pingus/bridger", *resource);
  pingu[dead]      = CL_Res_Surface::load("Pingus/dead", *resource);

  /*walker = CL_Res_Surface::load("Game/walker", *resource);
    fall_sprite = CL_Res_Surface::load("Game/faller", *resource);
    explo = CL_Res_Surface::load("Game/explo", *resource);

    if (!walker) {
    cerr << "Couldn't not load \"Game/walker\"" << endl;
    exit(EXIT_FAILURE);
    }
    if (!fall_sprite) {
    cerr << "Couldn't not load \"Game/faller\"" << endl;
    exit(EXIT_FAILURE);
    }
    if (!explo) {
    cerr << "Couldn't not load \"Game/explo\"" << endl;
    exit(EXIT_FAILURE);
    }
  */
}

int
Pingu::get_x()
{
  return x_pos;
}

int
Pingu::get_y()
{
  return y_pos;
}

void
Pingu::set_action(Action act)
{
  action = act;

  if (action != blocker) {
    colmap->del_blocker(blocker_id);
  }

  switch (action) {
  case bridger:
    break;
  case exploder:
    break;
  case blocker:
    if (rel_getpixel(0,-1) != 0) {
      blocker_id = colmap->set_blocker(x_pos, y_pos);
      anim_pos.set_size(6);
    } else {
      action = walker;
    }
    break;
  case floater:
    anim_pos.set_size(8);
    break;
  default:
    cout << "Unknown Action!" << endl;
    break;
  }
}

bool
Pingu::is_over(int x, int y)
{
  if (!alive)
    return false;

  if (x_pos - 16 < x && x_pos + 16 > x &&
      y_pos - 32 < y && y_pos > y) {
    return true;
  } else {
    return false;
  }
}

/* EOF */
